EllipseMorph subclass: #EyeMorph
	instanceVariableNames: 'iris '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Gestures'!

!EyeMorph methodsFor: 'initialization' stamp: 'len 9/6/1999 23:36'!
initialize
	super initialize.
	self color: (Color r: 1.0 g: 0.968 b: 0.935).
	self borderColor: Color black; borderWidth: 1.
	self extent: 30 @ 37.
	self addMorphFront: (iris _ EllipseMorph new extent: 6 @ 6; borderWidth: 0; color: Color black).
	self lookAtFront! !


!EyeMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 15:40'!
iris
	^ iris! !


!EyeMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:25'!
closeEyelid
	self iris delete.
	self position: self position + (0 @ (self extent y // 2)).
	self extent: self extent x @ 2! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/22/1999 15:57'!
dilate: amount
	| irisCenter |
	irisCenter _ self iris center.
	self iris extent: self iris extent * amount.
	self iris position: irisCenter - self iris center + self iris position! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/24/1999 00:48'!
lookAt: aPoint
	| theta scale |
	(self containsPoint: aPoint) ifTrue: [self iris align: iris center with: aPoint. ^ self].
	theta _ (aPoint - self center) theta.
	scale _ (aPoint - self center) r / 100.0 min: 1.0.
	self iris align: self iris center with: self center + (theta cos @ theta sin * self extent / 3.0 * scale) rounded! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:03'!
lookAtFront
	self iris position: self center - self iris center + self iris position! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:23'!
lookAtMorph: aMorph
	self lookAt: aMorph center! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/23/1999 22:49'!
openEyelid
	self extent: self extent x @ (self extent x * 37.0 / 30.0) rounded.
	self position: self position - (0 @ (self extent y // 2)).
	self addMorphFront: self iris! !

!EyeMorph methodsFor: 'actions' stamp: 'len 8/24/1999 01:18'!
openness: aNumber
	| previousCenter |
	previousCenter _ self center.
	self extent: self extent x @ (self extent x * 37.0 / 30.0 * aNumber) rounded.
	self align: self center with: previousCenter.
	(self containsPoint: self iris center) ifFalse: [self lookAtFront]! !


Morph subclass: #FaceMorph
	instanceVariableNames: 'leftEye leftEyebrow rightEye rightEyebrow lips '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Gestures'!

!FaceMorph methodsFor: 'initialization' stamp: 'len 8/22/1999 23:27'!
initialize
	super initialize.
	color _ Color transparent.
	self addMorph: (leftEye _ EyeMorph new).
	self addMorph: (rightEye _ EyeMorph new).
	self addMorph: (lips _ LipsMorph new).
	leftEye position: self position.
	rightEye position: leftEye extent x @ 0 + leftEye position.
	lips position: (0 @ 20 + (leftEye bottomRight + rightEye bottomLeft - lips extent // 2)).
	self bounds: self fullBounds! !


!FaceMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 22:21'!
leftEye
	^ leftEye! !

!FaceMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 19:47'!
lips
	^ lips! !

!FaceMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 22:21'!
rightEye
	^ rightEye! !


!FaceMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:23'!
closeEyelids
	leftEye closeEyelid.
	rightEye closeEyelid! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/24/1999 01:18'!
grin
	self leftEye openness: (0.2 to: 1.0 by: 0.1) atRandom.
	self rightEye openness: (0.2 to: 1.0 by: 0.1) atRandom.
	self lips grin! !

!FaceMorph methodsFor: 'actions' stamp: 'len 9/7/1999 02:29'!
happy
	self lips smile! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:00'!
hideTonge
	self lips hideTonge! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:21'!
lookAt: aPoint
	self leftEye lookAt: aPoint.
	self rightEye lookAt: aPoint! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/23/1999 22:51'!
lookAtFront
	self leftEye lookAtFront.
	self rightEye lookAtFront! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/24/1999 01:05'!
lookAtHand
	| hand |
	self isInWorld ifFalse: [^ self].
	hand _ (self world activeHand) ifNil: [self world primaryHand].
	self lookAtMorph: hand! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/24/1999 01:05'!
lookAtMorph: aMorph
	self leftEye lookAtMorph: aMorph.
	self rightEye lookAtMorph: aMorph! !

!FaceMorph methodsFor: 'actions' stamp: 'len 9/7/1999 02:25'!
neutral
	self lips neutral! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:23'!
openEyelids
	leftEye openEyelid.
	rightEye openEyelid! !

!FaceMorph methodsFor: 'actions' stamp: 'len 8/22/1999 23:18'!
say: aString
	self lips showBalloon: aString! !


!FaceMorph methodsFor: 'geometry' stamp: 'len 8/24/1999 01:27'!
mustachePosition
	^ self nosePosition + self lips center // 2! !

!FaceMorph methodsFor: 'geometry' stamp: 'len 8/24/1999 01:26'!
nosePosition
	^ self center * 2 + self lips center // 3! !


!FaceMorph methodsFor: 'drawing' stamp: 'len 8/22/1999 21:56'!
drawNoseOn: aCanvas
	| nosePosition |
	nosePosition _ self center * 2 + self lips center // 3.
	aCanvas fillOval: (nosePosition- (3@0) extent: 2 @ 2) color: Color black.
	aCanvas fillOval: (nosePosition + (3@0) extent: 2 @ 2) color: Color black! !

!FaceMorph methodsFor: 'drawing' stamp: 'len 8/22/1999 19:02'!
drawOn: aCanvas
	super drawOn: aCanvas.
	self drawNoseOn: aCanvas! !


!FaceMorph methodsFor: 'stepping and presenter' stamp: 'len 8/24/1999 01:22'!
step
	| amount |
	super step.
	10 atRandom = 1
		ifTrue: [[self lips perform: #(smile horror surprise sad grin) atRandom.
				 (Delay forMilliseconds: 2000 atRandom) wait.
				 self lips perform: #(neutral neutral smile sad) atRandom] fork].
	5 atRandom = 1
		ifTrue: [[self closeEyelids.
				 (Delay forMilliseconds: 180) wait.
				 self openEyelids.
				 2 atRandom = 1 ifTrue: [self lookAtFront]] fork.
				^ self].
	"20 atRandom = 1 ifTrue: [(self perform: #(leftEye rightEye) atRandom) closeEyelid]."
	20 atRandom = 1 ifTrue: [amount _ (0.2 to: 1.0 by: 0.01) atRandom.
							 self leftEye openness: amount. self rightEye openness: amount].
	3 atRandom = 1 ifTrue: [self lookAtHand. ^ self].
	3 atRandom = 1 ifTrue: [self lookAtFront. ^ self].
	3 atRandom = 1 ifTrue: [self lookAtMorph: self world submorphs atRandom]! !

!FaceMorph methodsFor: 'stepping and presenter' stamp: 'len 9/13/1999 00:18'!
stepTime
	^ 1000! !


Voice subclass: #GesturalVoice
	instanceVariableNames: 'head '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Gestures'!
!GesturalVoice commentStamp: '<historical>' prior: 0!
My instances are speaking voices with a head that acts in response to gestural events.!


!GesturalVoice methodsFor: 'accessing' stamp: 'len 8/28/1999 23:07'!
face
	^ self head face! !

!GesturalVoice methodsFor: 'accessing' stamp: 'len 8/28/1999 23:06'!
head
	^ head! !

!GesturalVoice methodsFor: 'accessing' stamp: 'len 9/28/1999 02:16'!
head: aHeadMorph
	head notNil ifTrue: [aHeadMorph position: head position. head delete].
	head _ aHeadMorph! !

!GesturalVoice methodsFor: 'accessing' stamp: 'len 8/28/1999 23:06'!
lips
	^ self face lips! !

!GesturalVoice methodsFor: 'accessing' stamp: 'len 9/28/1999 02:16'!
newHead
	| m |
	m _ HeadMorph new.
	self head: m.
	m openInWorld.
	^ m! !


!GesturalVoice methodsFor: 'playing' stamp: 'len 9/7/1999 01:41'!
playGesturalEvent: event at: time
	self head playEvent: event at: time! !

!GesturalVoice methodsFor: 'playing' stamp: 'len 9/6/1999 00:46'!
playPhoneticEvent: event at: time
	(TalkGesturalEvent new phoneme: event phoneme) playOn: self at: time! !


EllipseMorph subclass: #HeadMorph
	instanceVariableNames: 'face queue '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Gestures'!

!HeadMorph methodsFor: 'initialization' stamp: 'len 9/26/1999 17:20'!
initialize
	super initialize.
	self color: self randomSkinColor.
	self face: FaceMorph new.
	self extent: self face extent * (1.5 @ 1.7).
	self face align: self face center with: self center + (0 @ self height // 10).
	self addRandomFurnitures.
	queue _ SharedQueue new! !


!HeadMorph methodsFor: 'accessing-private' stamp: 'len 9/6/1999 00:43'!
queue
	^ queue! !


!HeadMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 23:25'!
face
	^ face! !

!HeadMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 23:25'!
face: aFaceMorph
	face notNil ifTrue: [face delete].
	self addMorphFront: (face _ aFaceMorph)! !

!HeadMorph methodsFor: 'accessing' stamp: 'len 9/7/1999 01:38'!
playEvent: event at: time
	self queue nextPut: time -> event! !


!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 00:19'!
addAfroHair
	| hair |
	hair _ CurveMorph
		vertices: {115@4. 144@20. 166@79. 132@131. 116@93. 88@85. 54@94. 40@134. 2@79. 31@16. 79@1}
		color: self randomHairColor
		borderWidth: 1
		borderColor: Color black.
	hair extent: (hair extent * (self width / hair width * 1.9)) rounded.
	hair align: hair center with: self center x @ self top.
	self addMorphFront: hair! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 9/13/1999 00:51'!
addBeret
	| beret pompon |
	beret _ CurveMorph
		vertices: {66@1. 90@14. 106@22. 114@35. 98@43. 55@35. 20@46. 2@38. 8@23. 23@13. 39@7}
		color: Color random
		borderWidth: 1
		borderColor: Color black.
	beret extent: (beret extent * (self width / beret width * 4 / 3)) rounded.
	beret align: beret center x @ beret bottom with: self center x @ self face top.
	pompon _ EllipseMorph new color: beret color; borderWidth: 1; borderColor: Color black; extent: beret height // 2.
	pompon align: pompon center with: beret center x @ beret top.
	beret addMorphFront: pompon.
	self addMorphFront: beret! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 02:26'!
addEars
	| leftEar rightEar |
	leftEar _ EllipseMorph new color: self color; extent: self height // 10 @ (self height // 7).
	rightEar _ leftEar copy.
	leftEar align: leftEar center with: self left @ self center y.
	rightEar align: rightEar center with: self right @ self center y.
	self addMorphBack: leftEar; addMorphBack: rightEar! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/23/1999 12:57'!
addGlasses
	| glass glass2 diameter |
	diameter _ self face leftEye height * 2 // 3.
	glass _ EllipseMorph new extent: diameter @ diameter; color: (Color yellow alpha: 0.5).
	glass2 _ glass copy.
	glass align: glass center with: self face leftEye center.
	glass2 align: glass2 center with: self face rightEye center.
	self addMorph: glass; addMorph: glass2! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/23/1999 23:33'!
addHighHat
	| hat |
	hat _ CurveMorph
		vertices: {70@3. 98@11. 94@46. 112@50. 96@58. 53@50. 18@61. 2@58. 24@48. 30@6. 47@6}
		color: Color random
		borderWidth: 1
		borderColor: Color black.
	hat extent: (hat extent * (self width / hat width * 4 / 3)) rounded.
	hat align: hat center x @ hat bottom with: self center x @ self face top.
	self addMorphFront: hat! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 01:40'!
addLargeMustache
	| mustache |
	mustache _ CurveMorph
		vertices: {48@4. 75@3. 93@15. 48@9. 3@19. 17@5}
		color: self randomHairColor
		borderWidth: 1
		borderColor: Color black.
	mustache extent: (mustache extent * (self width / mustache width)) rounded.
	mustache align: mustache center with: self face mustachePosition.
	self addMorphFront: mustache! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 9/13/1999 00:52'!
addRandomFurnitures
	self color: self randomSkinColor.
	self perform: #(yourself addBeret addHighHat addAfroHair addShortHair addSpikyHair) atRandom.
	self perform: #(yourself yourself addShortMustache) atRandom! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 00:19'!
addShortHair
	| hair |
	hair _ CurveMorph
		vertices: {81@3. 101@22. 105@48. 93@65. 76@32. 54@32. 28@35. 11@64. 2@52. 10@15. 45@2}
		color: self randomHairColor
		borderWidth: 1
		borderColor: Color black.
	hair extent: (hair extent * (self width / hair width * 1.15)) rounded.
	hair align: hair center x @ (hair top * 4 + hair bottom // 5) with: self center x @ self top.
	self addMorphFront: hair! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 01:39'!
addShortMustache
	| mustache |
	mustache _ CurveMorph
		vertices: {29@1. 54@14. 30@11. 1@15}
		color: self randomHairColor
		borderWidth: 1
		borderColor: Color black.
	mustache extent: (mustache extent * (self width / mustache width * 0.5)) rounded.
	mustache align: mustache center with: self face mustachePosition.
	self addMorphFront: mustache! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 02:14'!
addSpikyHair
	| hair |
	hair _ PolygonMorph
		vertices: {83@3. 81@30. 91@27. 111@23. 97@32. 112@37. 99@45. 114@52. 95@53. 55@43. 10@50. 1@40. 14@40. 8@26. 24@37. 15@11. 29@29. 30@16. 36@30. 41@6. 49@31. 54@8. 61@32. 64@1. 70@27}
		color: self randomHairColor
		borderWidth: 1
		borderColor: Color black.
	hair extent: (hair extent * (self width / hair width * 1.15)) rounded.
	hair align: hair center with: self center x @ self top.
	self addMorphFront: hair! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/29/1999 02:53'!
addWhiteHat
	| stage1 stage2 |
	stage1 _ CurveMorph
		vertices: {18@1. 93@18. 91@45. 8@40}
		color: (Color r: 1.0 g: 0.968 b: 0.935)
		borderWidth: 1
		borderColor: Color black.
	stage1 extent: (stage1 extent * (self width / stage1 width * 1.20)) rounded.

	stage2 _ CurveMorph
		vertices: {27@7. 81@5. 111@34. 11@28}
		color: stage1 color
		borderWidth: 1
		borderColor: Color black.
	stage2 extent: (stage2 extent * (self width / stage2 width * 1.20)) rounded.

	stage1 align: stage1 center with: self center x @ self top.
	stage2 align: stage2 center with: stage1 center x @ stage1 top.

	stage1 addMorphFront: stage2.
	self addMorphFront: stage1! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/24/1999 01:51'!
randomHairColor
	| hairColors |
	hairColors _ {Color r: 0.613 g: 0.161 b: 0.0. "red"
		Color r: 0.323 g: 0.226 b: 0.0. "dark brown"
		Color r: 0.774 g: 0.548 b: 0.0. "light brown"
		Color r: 0.968 g: 0.871 b: 0.0. "yellow"
		Color r: 0.581 g: 0.581 b: 0.581. "gray"
		Color black}.
	self submorphs do: [ :each | (hairColors includes: each color) ifTrue: [^ each color]].
	^ hairColors atRandom! !

!HeadMorph methodsFor: 'furnitures' stamp: 'len 8/29/1999 22:27'!
randomSkinColor
	^ {Color r: 0.258 g: 0.161 b: 0.0.
		Color r: 0.452 g: 0.258 b: 0.0.
		Color r: 0.516 g: 0.323 b: 0.0.
		Color r: 1.0 g: 0.935 b: 0.645.
		Color r: 1.0 g: 0.806 b: 0.548} atRandom! !


!HeadMorph methodsFor: 'stepping and presenter' stamp: 'len 9/26/1999 17:25'!
step
	| now |
	super step.
	now _ Time millisecondClockValue.
	[queue isEmpty not and: [now >= queue peek key]]
		whileTrue: [queue next value actOn: self].
	self face lips updateShape! !

!HeadMorph methodsFor: 'stepping and presenter' stamp: 'len 9/6/1999 23:30'!
stepTime
	^ 0! !


PolygonMorph subclass: #LipsMorph
	instanceVariableNames: 'newVertices newScale '
	classVariableNames: 'PhoneticArticulations '
	poolDictionaries: ''
	category: 'Speech-Gestures'!

!LipsMorph methodsFor: 'initialization' stamp: 'di 9/7/2000 17:22'!
initialize
	super initialize.
 	self beSmoothCurve.
	vertices _ {11@3. 35@1. 60@5. 67@17. 34@24. 3@17}.
	color _ Color black. "red darker."
	borderColor _ Color black.
	borderWidth _ 1.
	closed _ true.
	self neutral; updateShape! !


!LipsMorph methodsFor: 'accessing' stamp: 'len 8/22/1999 17:54'!
verticesString
	| stream first |
	stream _ WriteStream with: ''.
	stream nextPut: ${.
	first _ true.
	vertices do: [ :each |
		first ifFalse: [stream nextPutAll: '. '].
		stream print: (each - self position) rounded.
		first _ false].
	stream nextPut: $}.
	^ stream contents! !


!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 23:17'!
articulate: aPhoneme
	self newVerticesCentered: (PhoneticArticulations at: aPhoneme ifAbsent: [^ self]) scaled: 0.5! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/24/1999 01:13'!
grin
	self newVerticesCentered: {17@5. 30@7. 33@12. 16@14. 2@10. 5@2}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/24/1999 00:11'!
hideTongue
	self submorphs do: [ :each | each delete]! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:14'!
horror
	self newVerticesCentered: {21@3. 37@5. 36@19. 19@19. 3@19. 5@4}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:15'!
neutral
	self newVerticesCentered: {13@1. 22@4. 12@6. 1@4}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:34'!
newVerticesCentered: anArray
	self newVerticesCentered: anArray scaled: 0.5! !

!LipsMorph methodsFor: 'actions' stamp: 'len 9/6/1999 01:35'!
newVerticesCentered: anArray scaled: aNumber
	newVertices _ anArray.
	newScale _ aNumber! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 17:32'!
openness: amount
	self newVerticesCentered: {40@-3. 74@8. 59@ (21 * amount). 38@ (20 * amount). 22@ (21 * amount). 3@8}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:11'!
sad
	self newVerticesCentered: {26@4. 50@10. 41@4. 27@2. 14@3. 1@10}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/24/1999 00:10'!
showTongue
	| tongue |
	self hideTongue.
	tongue _ CurveMorph vertices: {10@2. 21@5. 16@23. 10@27. 5@23. 2@4} color: Color red borderWidth: 0 borderColor: Color black.
	tongue position: self center - (10 @ 0).
	self addMorphFront: tongue! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 19:12'!
smile
	self newVerticesCentered: {26@11. 43@7. 51@2. 44@12. 26@19. 8@14. 2@4. 12@9}! !

!LipsMorph methodsFor: 'actions' stamp: 'len 8/22/1999 22:34'!
surprise
	self newVerticesCentered:  {22@3. 38@7. 37@21. 20@26. 4@21. 6@6}! !

!LipsMorph methodsFor: 'actions' stamp: 'di 9/8/2000 10:44'!
updateShape
	| center median |
	newVertices isNil ifTrue: [^ self].
	median _ 0 @ 0.
	newVertices do: [ :each | median _ median + each].
	median _ median / newVertices size.
	center _ self center.
	self setVertices: (newVertices collect: [ :each | (each - median) * newScale + median]).
	self position: self position - self center + center.
	newVertices _ nil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LipsMorph class
	instanceVariableNames: ''!

!LipsMorph class methodsFor: 'class initialization' stamp: 'len 8/22/1999 18:28'!
initialize
	"
	LipsMorph initialize
	"
	| o u a e i m n p t s |
	a _ {22@2. 39@4. 45@16. 23@23. 2@15. 9@4}.
	e _ {23@2. 41@3. 45@15. 21@19. 2@14. 6@3}.
	i _ {21@2. 40@4. 45@14. 23@16. 2@13. 8@4}.
	o _ {18@1. 31@5. 31@20. 16@24. 3@19. 5@5}.
	u _ {14@1. 23@6. 22@16. 11@20. 2@14. 3@4}.
	m _ {17@2. 31@2. 27@4. 15@3. 2@4. 6@2}.
	n _ {20@3. 33@2. 39@8. 19@9. 2@8. 7@2}.
	p _ {7@1. 16@3. 12@5. 6@6. 1@4}.
	t _ {14@2. 29@3. 21@7. 12@8. 1@3}.
	s _ {19@2. 32@4. 35@10. 18@13. 2@10. 9@3}.
	PhoneticArticulations _ Dictionary new.
	"Default"
	PhonemeSet arpabet do: [ :each | PhoneticArticulations at: each put: p].
	"Vowels"
	PhonemeSet arpabet do: [ :each |
		each name first = $a ifTrue: [PhoneticArticulations at: each put: a].
		each name first = $e ifTrue: [PhoneticArticulations at: each put: e].
		each name first = $i ifTrue: [PhoneticArticulations at: each put: i].
		each name first = $o ifTrue: [PhoneticArticulations at: each put: o].
		each name first = $u ifTrue: [PhoneticArticulations at: each put: u].
		each name first = $w ifTrue: [PhoneticArticulations at: each put: u]].
	"Particulars"
	PhoneticArticulations
		at: (PhonemeSet arpabet at: 'm') put: m;
		at: (PhonemeSet arpabet at: 'n') put: n;
		at: (PhonemeSet arpabet at: 't') put: t;
		at: (PhonemeSet arpabet at: 's') put: s;
		at: (PhonemeSet arpabet at: 'sh') put: s;
		at: (PhonemeSet arpabet at: 'sh') put: s;
		at: (PhonemeSet arpabet at: 'zh') put: s;
		at: (PhonemeSet arpabet at: 'th') put: s;
		at: (PhonemeSet arpabet at: 'jh') put: s;
		at: (PhonemeSet arpabet at: 'dh') put: s;
		at: (PhonemeSet arpabet at: 'd') put: s;
		at: (PhonemeSet arpabet at: 'z') put: s! !

LipsMorph initialize!