Object subclass: #CosineInterpolator
	instanceVariableNames: 'origin points stack '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Support'!

!CosineInterpolator methodsFor: 'initialization' stamp: 'len 12/13/1999 02:42'!
initialize
	points _ SortedCollection new.
	stack _ SortedCollection new.
	origin _ 0! !


!CosineInterpolator methodsFor: 'accessing' stamp: 'len 12/14/1999 00:49'!
at: time
	"Answer the value of the receiver at a given time. (Do linear interpolation.)"
	^ self cosineAt: time + self origin! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 11/24/1999 01:59'!
at: time put: value
	self points add: time + self origin -> value.
	^ value! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 12/4/1999 17:30'!
commit
	self cleanBetween: stack first key and: stack last key.
	self points addAll: stack.
	stack _ SortedCollection new! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 12/4/1999 17:22'!
duration
	^ self points last key - self points first key! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 11/24/1999 01:59'!
origin
	^ origin! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 11/24/1999 01:59'!
origin: aNumber
	origin _ aNumber! !

!CosineInterpolator methodsFor: 'accessing' stamp: 'len 12/4/1999 17:20'!
x: x y: y
	stack add: x + self origin -> y! !


!CosineInterpolator methodsFor: 'private' stamp: 'len 11/23/1999 01:08'!
cleanBetween: start and: end
	self points: (self points reject: [ :each | each key between: start and: end])! !

!CosineInterpolator methodsFor: 'private' stamp: 'len 11/23/1999 00:42'!
cosineAt: time
	"Answer the value of the receiver at a given time. (Do cosine interpolation.)"
	| xVal count x1 x2 y1 y2 |
	points isNil ifTrue: [^ nil].
	xVal _ points first key.
	count _ 1.
	[xVal < time]
		whileTrue: [count _ count + 1.
					count > points size ifTrue: [^ points last value].
					xVal _ (points at: count) key].
	xVal = time ifTrue: [^ (points at: count) value].
	count = 1 ifTrue: [^ points first value].
	x1 _ (points at: count - 1) key.
	x2 _ (points at: count) key.
	y1 _ (points at: count - 1) value.
	y2 _ (points at: count) value.
	^ ((time - x1 / (x2 - x1) * Float pi) cos - 1 / -2.0) * (y2 - y1) + y1! !

!CosineInterpolator methodsFor: 'private' stamp: 'len 12/4/1999 15:54'!
linearAt: time
	"Answer the value of the receiver at a given time. (Do linear interpolation.)"
	| xVal count x1 x2 y1 y2 |
	points isNil ifTrue: [^ nil].
	xVal _ points first key.
	count _ 1.
	[xVal < time]
		whileTrue: [count _ count + 1.
					count > points size ifTrue: [^ points last value].
					xVal _ (points at: count) key].
	xVal = time ifTrue: [^ (points at: count) value].
	count = 1 ifTrue: [^ points first value].
	x1 _ (points at: count - 1) key.
	x2 _ (points at: count) key.
	y1 _ (points at: count - 1) value.
	y2 _ (points at: count) value.
	^ (time - x1) / (x2 - x1) * (y2 - y1) + y1! !

!CosineInterpolator methodsFor: 'private' stamp: 'len 12/4/1999 17:29'!
points
	^ points! !

!CosineInterpolator methodsFor: 'private' stamp: 'len 11/23/1999 00:41'!
points: aCollection
	points _ aCollection! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

CosineInterpolator class
	instanceVariableNames: ''!

!CosineInterpolator class methodsFor: 'instance creation' stamp: 'len 12/13/1999 02:15'!
fromArray: anArray
	| answer |
	answer _ self new.
	1 to: anArray size by: 2 do: [ :each | answer at: (anArray at: each) put: (anArray at: each + 1)].
	^ answer! !

!CosineInterpolator class methodsFor: 'instance creation' stamp: 'len 12/4/1999 17:29'!
new
	^ super new initialize! !


Object subclass: #DECTalkReader
	instanceVariableNames: 'stream phonemes durations events currentDuration currentPitch f0Contour '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Support'!

!DECTalkReader methodsFor: 'initialization' stamp: 'len 12/24/1999 03:20'!
initialize
	phonemes _ PhonemeSet dectalkToArpabet.
	events _ CompositeEvent new.
	currentDuration _ 80.
	currentPitch _ 100.
	f0Contour _ CosineInterpolator new.
	durations _ Dictionary new.
	#(
	('ae'	230.0	80.0)
	('aa'	240.0	100.0)
	('ax'	120.0	60.0)
	('er'	180.0	80.0)
	('ay'	250.0	150.0)
	('aw'	240.0	100.0)
	('b'		85.0		60.0)
	('ch'	70.0		50.0)
	('d'		75.0		50.0)
	('dh'	50.0		30.0)
	('eh'	150.0	70.0)
	('ea'	270.0	130.0)
	('ey'	180.0	100.0)
	('f'		100.0	80.0)
	('g'		80.0		60.0)
	('hh'	80.0		20.0)
	('ih'	135.0	40.0)
	('ia'	230.0	100.0)
	('iy'	155.0	55.0)
	('jh'	70.0		50.0)
	('k'		80.0		60.0)
	('l'		80.0		40.0)
	('m'		70.0		60.0)
	('n'		60.0		50.0)
	('ng'	95.0		60.0)
"	('oh'	240.0	130.0)"
	('oy'	280.0	150.0)
	('ao'	240.0	130.0)
	('ow'	220.0	80.0)
	('p'		90.0		50.0)
	('r'		80.0		30.0)
	('s'		105.0	60.0)
	('sh'	105.0	80.0)
	('t'		75.0		50.0)
	('th'	90.0		60.0)
	('uh'	210.0	70.0)
	('ua'	230.0	110.0)
	('ah'	160.0	60.0)
	('uw'	230.0	150.0)
	('v'		60.0		40.0)
	('w'		80.0		60.0)
	('y'		80.0		40.0)
	('z'		75.0		40.0)
	('zh'	70.0		40.0)
	('sil'	100.0	100.0)) do: [ :each |
		durations at: (PhonemeSet arpabet at: each first) put: each second / 1000.0]! !


!DECTalkReader methodsFor: 'accessing' stamp: 'len 12/20/1999 03:28'!
defaultDurationFor: aPhoneme
	^ durations at: aPhoneme ifAbsent: [0.080]! !

!DECTalkReader methodsFor: 'accessing' stamp: 'len 12/20/1999 03:04'!
events
	^ events! !

!DECTalkReader methodsFor: 'accessing' stamp: 'len 12/20/1999 03:04'!
phonemes
	^ phonemes! !

!DECTalkReader methodsFor: 'accessing' stamp: 'len 12/20/1999 03:04'!
stream: aStream
	stream _ aStream! !


!DECTalkReader methodsFor: 'reading' stamp: 'len 12/20/1999 03:37'!
addPitches
	| offset |
	offset _ 0.0.
	events do: [ :each |
		each pitchPoints: (self pitchesBetween: offset and: offset + each duration).
		offset _ offset + each duration].! !

!DECTalkReader methodsFor: 'reading' stamp: 'len 12/20/1999 04:31'!
nextPhoneme
	| try try2 phon |
	try _ stream next asString.
	(',.;-' includes: try first) ifTrue: [^ phonemes at: 'sil'].
	try2 _ try, stream peek asString.
	(phon _ phonemes at: try2 ifAbsent: []) notNil ifTrue: [stream next. ^ phon].
	^ phonemes at: try! !

!DECTalkReader methodsFor: 'reading' stamp: 'len 12/20/1999 03:37'!
pitchesBetween: t1 and: t2
	| step |
	step _ (t2 - t1 / 0.035) asInteger + 1. "step small enough"
	^ (t1 to: t2 by: t2 - t1 / step) collect: [ :each | each - t1 @ (f0Contour at: each)]! !

!DECTalkReader methodsFor: 'reading' stamp: 'len 12/24/1999 05:35'!
read
	| phoneme time |
	time _ 0.
	[stream skipSeparators; atEnd]
		whileFalse: [phoneme _ self nextPhoneme.
					currentDuration _ self defaultDurationFor: phoneme.
					stream peek = $< ifTrue: [self readPitchAndDuration].
					f0Contour at: time + (currentDuration / 2.0 min: 0.1) put: currentPitch.
					time _ time + currentDuration.
					f0Contour at: time put: currentPitch.
					events add: (PhoneticEvent new phoneme: phoneme; duration: currentDuration; loudness: 1.0)].
	self addPitches! !

!DECTalkReader methodsFor: 'reading' stamp: 'len 12/24/1999 03:17'!
readPitchAndDuration
	| tokens code |
	stream next.
	tokens _ (stream upTo: $>) findTokens: ','.
	currentDuration _ tokens first asNumber / 1000.0.
	tokens size > 1 ifFalse: [^ self].
	code _ tokens last asNumber.
	currentPitch _ code > "37" 64 ifTrue: [code] ifFalse: [AbstractSound pitchForMIDIKey: 35 + code]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DECTalkReader class
	instanceVariableNames: ''!

!DECTalkReader class methodsFor: 'instance creation' stamp: 'len 12/20/1999 03:52'!
eventsFromStream: aStream
	^ self new stream: aStream; read; events! !

!DECTalkReader class methodsFor: 'instance creation' stamp: 'len 12/20/1999 03:52'!
eventsFromString: aString
	^ self eventsFromStream: (ReadStream on: aString)! !

!DECTalkReader class methodsFor: 'instance creation' stamp: 'len 12/20/1999 03:52'!
new
	^ super new initialize! !


!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:14'!
daisy
	"
	DECTalkReader daisy playOn: KlattVoice new delayed: 10000
	"
	^ self eventsFromString: '_<50,22>dey<400,22>ziy<400,19>dey<400,15>ziy<400,10>
gih<200,12>vmiy<200,14>yurr<200,15>ae<400,12>nsax<200,15>
rduw<400,10>.
ay<400,17>mhxae<400,22>fkrey<400,19>ziy<400,15>ao<200,12>
lfao<200,14>rdhax<200,15>lah<400,17>vao<200,19>vyu<400,17>.
ih<200,19>twow<200,20>ntbiy<200,19>ax<200,17>stay<400,22>
lih<200,19>shmae<200,17>rih<400,15>jh<50,15>.
ay<200,17>kae<400,19>ntax<200,15>fow<400,12>rdax<200,15>
kae<200,12>rih<400,10>jh<50,10>.
bah<200,10>tyu<400,15>lluh<200,19>kswiy<400,17>tah<200,10>
pao<400,15>ndhax<200,19>siy<400,17>t<50,17>.
ao<200,17>vax<200,19>bay<200,22>six<200,19>kel<200,15>
bih<400,17>ltfao<200,10>rtuw<800,15>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:48'!
flower
	"
	DECTalkReader flower playOn: KlattVoice new delayed: 15000
	"
	^ self eventsFromString: '_<25,22>ow<200,22>flaw<400,22>rax<200,20>vskao<400,18>
ao<200,18>tlae<800,13>nd<200,13>
weh<200,13>nwih<400,18>lwiy<200,22>siy<800,20>yu<200,20>
rlay<400,18>kax<200,20>geh<1600,22>n<25,22>
dhax<200,22>tfao<300,23>ao<100,22>tae<200,23>nday<400,25>
d<200,25>fao<800,18>r<25,18>
yu<200,13>rwiy<400,20>bih<200,20>t hxih<200,20>ih<200,18>
lae<200,20>ndgleh<400,22>nae<200,23>ndstuh<400,22>dax<200,20>
geh<600,18>nst hxih<800,13>m<200,13>
praw<200,22>deh<300,23>eh<100,22>dwax<200,23>rdzaa<400,25>
aa<200,25>rmih<800,18>ih<200,18>
ae<200,22>ndseh<200,23>eh<200,22>nt hxih<200,20>mhxow<300,22>
ow<100,20>ow<200,18>mwax<800,18>ax<200,18>rdtey<200,18>
thih<400,16>nxkax<200,20>geh<800,18>eh<400,18>n<200,18>
_<600,22>dhax<200,22>hxih<400,22>lzax<200,20>rbey<400,18>
rr<200,18>naw<800,13>
ae<200,13>ndao<400,18>tah<200,22>mliy<800,20>vzlay<200,20>
thih<400,18>kax<200,20>ndstih<800,22>ih<800,22>l<25,22>
ow<200,22>rlae<300,23>nddhax<100,22>tih<200,23>zlao<400,25>
ao<200,25>stnaw<800,18>
wih<200,13>chdhow<400,20>zsow<200,20>diy<200,20>ax<200,18>
lih<200,20>hxeh<400,22>ldhax<200,23>tstuh<400,22>dax<200,20>
geh<400,18>eh<200,18>nst hxih<800,13>m<200,13>
praw<200,22>deh<300,23>eh<100,22>dwax<200,23>rdzaa<400,25>
aa<200,25>rmih<800,18>ih<200,18>
ae<200,22>ndseh<200,23>eh<200,22>nt hxih<200,20>mhxow<300,22>
ow<100,20>ow<200,18>mwax<800,18>ax<200,18>rdtey<200,18>
thih<400,16>nxkax<200,20>geh<1200,18>n<200,18>
_<600,22>dhow<200,22>zdey<400,22>zax<200,20>rpae<400,18>
ae<200,18>stnaw<800,13>
ae<200,13>ndih<400,18>ndhax<200,22>pae<800,20>stdhey<200,20>
mah<400,18>strix<200,20>mey<800,22>ey<800,22>n<25,22>
bah<200,22>twiy<300,23>kae<100,22>nstih<200,23>lray<600,25>
znaw<800,18>
ae<200,13>ndbiy<400,20>dhax<200,20>ney<200,20>shax<200,18>
nax<200,20>geh<400,22>ndhax<200,23>tstuh<400,22>dax<200,20>
geh<600,18>nst hxih<800,13>m<200,13>
praw<200,22>deh<300,23>eh<100,22>dwax<200,23>rdzaa<400,25>
aa<200,25>rmih<800,18>ih<200,18>
ae<200,22>ndseh<200,23>eh<200,22>nt hxih<200,20>mhxow<300,22>
ow<100,20>ow<200,18>mwax<800,18>ax<200,18>rdtey<200,18>
thih<400,16>nxkax<200,20>geh<1200,18>n<200,18>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:43'!
great
	"
	(DECTalkReader great pitchBy: 0.5) playOn: (KlattVoice new tract: 19; flutter: 0.5) delayed: 10000
	"
	^ self eventsFromString: '_<50,20>ax<200,20>_<1>ax<500,22>_<10>ow<300,20>yxeh<1000,17>
say<200,15>mdhax<80,13>grey<1000,15>t_priy<200,17>iy<200,18>
teh<200,17>eh<200,15>ndrr<1600,13>
_<200,13>priy<200,13>teh<1000,22>ndih<200,22>nxdhae<100,22>
day<1000,18>mduh<200,20>ix<200,22>nweh<1600,20>l<600,20>
_<60,25>may<300,25>niy<1200,22>dix<200,22>zsah<1000,24>chay<200,22>
priy<200,24>teh<1000,25>ndtuh<200,22>_<10>uw<200,25>mah<1000,20>ch<100,20>
_<20,20>ay<300,20>mlow<300,20>neliy<800,17>bah<200,13>
tnow<1000,15>wah<200,13>nkae<200,15>nteh<1800,13>l<400,13>
_<50,20>ax<200,20>_<1>ax<500,22>_<1>ow<300,20>yxeh<1000,17>
say<200,15>mdhax<80,13>grey<1000,15>t_priy<200,17>iy<200,18>
teh<200,17>eh<200,15>ndrr<1800,13>
_<10,13>ah<200,13>drih<1000,22>ftih<50,22>nax<200,22>
wrr<1000,18>ldax<200,20>vmay<200,22>ax<200,22>_<1>ow<1400,20>n<600,20>
_<60,25>ay<300,25>pley<1100,22>dhax<200,22>gey<1000,24>
m<100,24>bah<200,22>tuh<200,24>may<1000,25>riy<200,22>ax<200,25>
lshey<600,20>m<400,20>
_<20,20>yu<200,20>vleh<200,20>ftmiy<800,17>tuw<200,13>
driy<800,15>mao<200,13>lah<200,15>low<1600,13>n<400,13>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:25'!
hawaii
	"
	DECTalkReader hawaii playOn: (KlattVoice new tract: 14.4) delayed: 10000
	"
	^ self eventsFromString: '_<300> naa<600,23> ay<300,23>t ae<300,22>nd yuw<1200,23> ,<600>
ae<600,24>nd
bluw<900,25> hxah<300,24> waa<940,23> aa<240,22> aa<240,21> iy<1200,20>
,<600> dhah<600,32> naa<600,33> ay<300,33>t ih<300,32>z hxeh<900,30>
veh<300,22>n liy<1200,25>
,<600> ae<300,30> ae<300,31>nd yu<900,32> aa<300,30>rx
hxeh<440,28> veh<440,28>n tuw<440,25> miy<2400,23> ,<600> lah<900,23>v
liy<300,22> yuw<1200,23> ,<600> ae<600,24>nd bluw<900,25> hxah<300,24>
waa<940,23>-aa<240,22>-aa<240,21>-iy<1200,20>
,<600> wih<600,32>dh ao<900,33>lx dhih<300,32>s lah<880,30> v<40,30>
liy<300,22> neh<1200,25>s
,<600> dheh<300,30> eh<300,31>rx shuh<900,32>d biy<300,27>
lah<4140,28> v<60,28> ,<600> kah<900,25>m wih<300,32>dh miy<1800,30> ,<600>
waa<400,28> ay<200,28>lx dhah<600,25> muw<300,28>n ih<300,25>z aa<300,28>n
dhah<300,25> siy<2400,23> ,<600> dhah<600,24> naa<600,25> ay<300,25>t
ih<300,32>z yxah<1200,30>nx ,<600> ae<600,28>nd sow<600,25>
aa<600,28>rx-wiy<4200,30> ,<600> driy<900,23>mz kah<300,22>m truw<1000,23>
uw<200,23> ,<600> ih<600,24>n
bluw<900,25> hxah<300,24> waa<940,23> aa<240,22> aa<240,21> iy<1200,20>
,<600> ae<600,32>nd maa<600,33> iy<300>n kuh<300,32>d ao<900,30>lx
kah<300,22>m truw<1200,25> ,<600> dhih<300,30> ih<300,31>s mae<900,32>
jhih<330,27>k naa<600,28> ay<350,28>t ah<350,27>v naa<600,28> ay<350,28>ts
,<40> wih<380,27>dh yuw<1000,28>-uw<600,455>-uw<1800,35>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 05:14'!
silentNightDuetExample
	"
	DECTalkReader silentNightDuetExample
	"

	| song1 song2 voice1 voice2 time |
	song1 _ DECTalkReader silentNightVoice1.
	song2 _ DECTalkReader silentNightVoice2.
	voice1 _ KlattVoice new tract: 14.4.
	voice2 _ KlattVoice new tract: 18.5; turbulence: 59.
	time _ Time millisecondClockValue + 30000. "give it 30 secounds for precomputing"
	song1 playOn: voice1 at: time.
	song2 playOn: voice2 at: time! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 05:14'!
silentNightDuetExample2
	"
	DECTalkReader silentNightDuetExample2
	"

	| song1 song2 voice1 voice2 time |
	song1 _ DECTalkReader silentNightVoice1 pitchBy: 0.5.
	song2 _ DECTalkReader silentNightVoice2 pitchBy: 0.5.
	voice1 _ KlattVoice new tract: 14.4.
	voice2 _ KlattVoice new tract: 18.5; turbulence: 59.
	time _ Time millisecondClockValue + 30000. "give it 30 secounds for precomputing"
	song1 playOn: voice1 at: time.
	song2 playOn: voice2 at: time! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 05:14'!
silentNightDuetExample3
	"
	DECTalkReader silentNightDuetExample3
	"

	| song1 song2 voice1 voice2 time |
	song1 _ DECTalkReader silentNightVoice1 pitchBy: 0.25.
	song2 _ DECTalkReader silentNightVoice2 pitchBy: 0.25.
	voice1 _ KlattVoice new tract: 18.5; turbulence: 59.
	voice2 _ KlattVoice new tract: 20; flutter: 0.5.
	time _ Time millisecondClockValue + 30000. "give it 30 secounds for precomputing"
	song1 playOn: voice1 at: time.
	song2 playOn: voice2 at: time! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 05:18'!
silentNightDuetExample4
	"
	DECTalkReader silentNightDuetExample4
	"

	| song1 song2 voice1 voice2 gestural1 gestural2 time |
	song1 _ DECTalkReader silentNightVoice1 pitchBy: 0.5.
	song2 _ DECTalkReader silentNightVoice2 pitchBy: 0.5.
	gestural1 _ GesturalVoice new.
	gestural1 newHead position: 1 @ 50.
	voice1 _ (KlattVoice new tract: 14.4) + gestural1.
	gestural2 _ GesturalVoice new.
	gestural2 newHead position: 150 @ 50.
	voice2 _ (KlattVoice new tract: 18.5; turbulence: 59) + gestural2.
	time _ Time millisecondClockValue + 30000. "give it 30 secounds for precomputing"
	song1 playOn: voice1 at: time.
	song2 playOn: voice2 at: time! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 04:58'!
silentNightVoice1
	"
	(DECTalkReader silentNightVoice1 pitchBy: 0.5) playOn: KlattVoice new delayed: 1000
	"

	^ self eventsFromString: 'sae<600,32>ay<200,34>leh<400,32>nt nae<600,29>ay<400>t.
hxow<600,32>ow<200,34>liy<400,32> nae<600,29>ay<400>t.
ao<600,39>l ih<200>z kaa<800,36>lm.
ao<600,37>l ih<200>z bray<800,32>t.
raw<600,34>nd yah<400>ng ver<600,37>er<200,36>jhah<400,34>n
mah<600,32>dher<200,32> ae<400>nd chah<600,29>ay<200>ld.
hxow<800,34>liy<400> ih<600,37>nfah<200,36>nt
sow<400,34> teh<600,32>nder<400,34> ae<400,32>nd may<600,29>ld.
sliy<600,39>p ah<400>n hxeh<400,42>vah<400,39>nliy<400,36> piy<1000,37>iy<800,41>s.
sliy<400,37>iy<400,32>p ah<400,29>n hxeh<400,32>vah<400,30>nliy<600,27> piy<1800,25>s.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/24/1999 05:06'!
silentNightVoice2
	"
	(DECTalkReader silentNightVoice2 pitchBy: 0.5) playOn: KlattVoice new delayed: 1000
	"

	^ self eventsFromString: 'sae<600,29>ay<200,30>leh<400,29>nt nae<600,25>ay<400>t.
hxow<600,29>ow<200,30>liy<400,29> nae<600,25>ay<400>t.
ao<600,30>l ih<200>z kaa<800,27>lm.
ao<600,29>l ih<200>z bray<800,29>t.
raw<600,30>nd yah<400>ng ver<600,34>er<200,32>jhah<400,30>n
mah<600,29>dher<200,30> ae<400,29>nd chah<600,25>ay<200>ld.
hxow<800,30>liy<400> ih<600,34>nfah<200,32>nt
sow<400,30> teh<600,29>nder<400,30> ae<400,29>nd may<600,25>ld.
sliy<600,30>p ah<400>n hxeh<400,27>vah<400,30>nliy<400,27> piy<1000,29>iy<800,32>s.
sliy<400,29>iy<400,29>p ah<400,25>n hxeh<400,24>vah<400,24>nliy<600,24> piy<1800,25>s.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:52'!
startrek
	"
	DECTalkReader startrek playOn: KlattVoice new delayed: 15000
	"
	^ self eventsFromString: '_<50,17>dhey<400,17>rsklih<100,17>nxao<100,17>nzao<100,17>
ndhax<100,17>staa<100,20>rbao<100,20>rdbaw<200,17>staa<100,18>
rbao<100,18>rdbaw<200,15>staa<100,17>rbao<100,17>rdbaw<200,13>.

dhey<100,17>rsklih<100,17>nxao<100,17>nzao<100,17>ndhax<100,17>
staa<100,20>rbao<100,20>rdbaw<200,17>staa<100,18>rbao<100,18>
rdbaw<200,15>jhih<400,13>m<50,13>.

_<50,17>ih<400,17>tslay<100,17>fjhih<100,17>mbah<100,17>
tnao<50,20>tax<50,20>zwiy<100,20>now<100,17>ih<200,17>
tnao<50,18>tax<50,18>zwiy<100,18>now<100,15>ih<200,15>
tnao<50,17>tax<50,17>zwiy<100,17>now<100,13>ih<200,13>
t<50,13>.

ih<100,17>tslay<100,17>fjhih<100,17>mbah<100,17>tnao<50,20>
tax<50,20>zwiy<100,20>now<100,17>ih<200,17>tnao<50,18>
tax<50,18>zwiy<100,18>now<100,15>ih<200,15>tkae<200,13>
ptix<200,13>n<50,13>.

_<50,17>ih<400,17>tswah<100,17>rsdhae<100,17>ndhae<100,17>
t_hxiy<100,20>zdeh<200,20>djhih<200,17>mdeh<200,18>djhih<200,15>
mdeh<200,17>djhih<200,13>m<50,13>.

ih<100,17>tswah<100,17>rsdhae<100,17>ndhae<100,17>t_hxiy<100,20>
zdeh<200,20>djhih<200,17>mdeh<200,18>djhih<200,15>mdeh<400,13>
d<50,13>.

_<50,17>wiy<400,17>kah<100,17>mih<100,17>npiy<200,17>
sshuh<100,20>tuh<100,20>kih<200,17>lshuh<100,18>tuh<100,18>
kih<200,15>lshuh<100,17>tuh<100,17>kih<200,13>l<50,13>.

wiy<100,17>kah<100,17>mih<100,17>npiy<200,17>sshuh<100,20>
tuh<100,20>kih<200,17>lshuh<100,18>tuh<100,18>kih<200,15>
lmeh<400,13>n<50,13>.

_<50,17>yxih<400,17>kaa<100,17>naa<100,17>chey<100,17>
njhdhax<50,17>lao<50,20>zax<100,20>fih<100,17>zih<100,17>
kslao<50,18>zax<100,18>fih<100,15>zih<100,15>kslao<50,17>
zax<100,17>fih<100,13>zih<100,13>ks<50,13>.

yxih<400,17>kaa<100,17>naa<100,17>chey<100,17>njhdhax<50,17>
lao<50,20>zax<100,20>fih<100,17>zih<100,17>kslao<50,18>
zax<100,18>fih<100,15>zih<100,15>kskaa<200,13>ptix<200,13>
n<50,13>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:52'!
startrek1
	"
	DECTalkReader startrek1 playOn: KlattVoice new delayed: 5000
	"
	^ self eventsFromString: '_<50,17>dhey<400,17>rsklih<100,17>nxao<100,17>nzao<100,17>
ndhax<100,17>staa<100,20>rbao<100,20>rdbaw<200,17>staa<100,18>
rbao<100,18>rdbaw<200,15>staa<100,17>rbao<100,17>rdbaw<200,13>.'! !

!DECTalkReader class methodsFor: 'examples' stamp: 'len 12/20/1999 04:38'!
vermont
	"
	(DECTalkReader vermont pitchBy: 0.5) playOn: (KlattVoice new tract: 18.5; turbulence: 59) delayed: 15000
	"
	^ self eventsFromString: 'peh<400,25> niy<400,23>z ih<400,20>n ah<400,18> striy<1200,20>m
,<400> fao<400,25> lih<400,23>nx liy<500,20>vz ,<100> ah<200,16>v 
sih<200,18> kah<200,20> mao<1000,12>rx ,<200> muw<400,20>n lay<400,18>t
ih<400,16>n vrr<400,13> maa<1200,16>nt ,<400> ay<400,25> siy<400,23> 
fih<400,20>nx grr<400,18> wey<1200,20>vz ,<400> skiy<400,25> trae<300,23>lxz  
,<100> aa<600,20>n ah<200,16> maw<200,18>n tih<200,20>n 
saa<800,12>-ay<200,12>d ,<200> snow<400,20> lay<400,18>t
ih<400,16>n vrr<400,13> maa<1200,16>nt ,<400> teh<200,15> lah<200,15>
grae<200,15>f key<400,15> bah<300,15>lxz ,<100> dhey<200,15> sih<200,15>nx 
daw<400,15>n dhah<200,15> hxay<200,15> wey<300,15> ,<100> ae<200,15>nd
trae<200,15> vuh<200,15>lx iy<200,15>ch beh<500,27>nd,<100> ih<200,25>n 
dhah<200,27> row<1200,24>d ,<400> piy<200,16> pah<200,16>lx hxuw<200,16>
miy<200,16>t ,<200> 
ih<400,16>n-dhih<200,16>s-row<200,16>-mae<400,16>n-tih<160,16>k ,<40> 
seh<200,16> tih<300,16>nx ,<100> aa<200,16>rx sow<200,16>
hxih<200,16>p nah<200,16> tay<400,28>zd ,<200> bay<200,26> dhah<200,28>
lah<900,25>v liy<700,24> ,<200> iy<400,25>v nih<400,23>nx sah<400,20>
mrr<400,18> briy<1200,20>z ,<400> wao<400,25>rx blih<400,23>nx ah<400,20>v
,<200> ah<200,16> meh<200,18> dow<200,20> laa<800,12>rxk ,<400> muw<400,20>n
lay<400,18>t ih<400,16>n vrr<400,13> maa<1300,16>nt ,<400>
iy<40,12>-yuw<280,12> ae<350,13>n day<420,16> ,<60> ae<340,20>nd
muw<380,25>n lay<340,27>t ,<100> ih<500,24>n vrr<540,26> maa<2000,23>nt.'! !


Object subclass: #PHOReader
	instanceVariableNames: 'stream phonemes events pitches time '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Speech-Support'!
!PHOReader commentStamp: '<historical>' prior: 0!
My instances read PHO files with lines of the form 'phoneme duration time0 pitch0 time1 pitch1 ...'. Time is in milliseconds, and pitch is in hertz. Files on this format are used as inputs for the MBROLA synthesizer, and there are lots of them available on the web. Here's an example:

_  120 0 105
m  60 33 105
E  70 42 102
r  50 20 108
I  50 20 125 100 142
k  100
r  50 80 137
I  50 80 121
s  70
m  50
@  90 33 111 88 108
s  90
!


!PHOReader methodsFor: 'initialization' stamp: 'len 6/15/1999 00:36'!
initialize
	events _ OrderedCollection new.
	pitches _ OrderedCollection new.
	time _ 0.0.
	phonemes _ PhonemeSet sampaToArpabet! !


!PHOReader methodsFor: 'accessing' stamp: 'len 9/27/1999 23:01'!
addPitches
	| offset |
	offset _ 0.0.
	events do: [ :each |
		each pitchPoints: (self pitchesBetween: offset and: offset + each duration).
		offset _ offset + each duration].! !

!PHOReader methodsFor: 'accessing' stamp: 'len 8/29/1999 02:16'!
events
	^ CompositeEvent new addAll: events; yourself! !

!PHOReader methodsFor: 'accessing' stamp: 'len 11/28/1999 04:00'!
nextEvent
	| line phonemeName phoneme duration answer ptime pitch |
	line _ ReadStream on: stream nextLine.
	phonemeName _ line upTo: Character space.
	phoneme _ phonemes at: phonemeName.
	[line peek isSeparator] whileTrue: [line next].
	duration _ (line upTo: Character space) asNumber / 1000.0.
	answer _ PhoneticEvent new phoneme: phoneme; duration: duration; loudness: 1.0.
	[line atEnd]
		whileFalse: [ptime _ (line upTo: Character space) asNumber * duration / 100.0.
					pitch _ (line upTo: Character space) asNumber asFloat.
					pitches add: time + ptime @ pitch].
	time _ time + duration.
	^ answer! !

!PHOReader methodsFor: 'accessing' stamp: 'len 6/15/1999 00:38'!
pitchAt: t
	"Answer the pitch of the receiver at a given time. (Do linear interpolation.)"
	| xVal count x1 x2 y1 y2 |
	xVal _ pitches first x.
	count _ 1.
	[xVal < t]
		whileTrue: [count _ count + 1.
					count > pitches size ifTrue: [^ pitches last y].
					xVal _ (pitches at: count) x].
	xVal = t ifTrue: [^ (pitches at: count) y].
	count = 1 ifTrue: [^ pitches first y].
	x1 _ (pitches at: count - 1) x.
	x2 _ (pitches at: count) x.
	y1 _ (pitches at: count - 1) y.
	y2 _ (pitches at: count) y.
	^ (t - x1) / (x2 - x1) * (y2 - y1) + y1! !

!PHOReader methodsFor: 'accessing' stamp: 'len 6/20/1999 23:57'!
pitchesBetween: t1 and: t2
	| step |
	step _ (t2 - t1 / 0.035) asInteger + 1. "step small enough"
	^ (t1 to: t2 by: t2 - t1 / step) collect: [ :each | each - t1 @ (self pitchAt: each)]! !

!PHOReader methodsFor: 'accessing' stamp: 'len 6/17/1999 01:45'!
plotPitch
	Utilities plot: ((0 to: time by: 0.050) collect: [ :each | self pitchAt: each])! !

!PHOReader methodsFor: 'accessing' stamp: 'len 6/15/1999 00:40'!
read
	stream reset.
	[stream atEnd] whileFalse: [events add: self nextEvent].
	self addPitches! !

!PHOReader methodsFor: 'accessing' stamp: 'len 6/15/1999 00:29'!
stream: aStream
	stream _ aStream! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

PHOReader class
	instanceVariableNames: ''!

!PHOReader class methodsFor: 'instance creation' stamp: 'len 6/15/1999 00:48'!
eventsFromStream: aStream
	^ self new stream: aStream; read; events! !

!PHOReader class methodsFor: 'instance creation' stamp: 'len 6/15/1999 00:49'!
eventsFromString: aString
	^ self eventsFromStream: (ReadStream on: aString)! !

!PHOReader class methodsFor: 'instance creation' stamp: 'len 6/15/1999 00:31'!
new
	^ super new initialize! !

!PHOReader class methodsFor: 'instance creation' stamp: 'len 6/17/1999 01:47'!
plotPitchFromStream: aStream
	^ self new stream: aStream; read; plotPitch! !

!PHOReader class methodsFor: 'instance creation' stamp: 'len 6/17/1999 01:47'!
plotPitchFromString: aString
	^ self plotPitchFromStream: (ReadStream on: aString)! !


!PHOReader class methodsFor: 'examples' stamp: 'len 9/13/1999 01:29'!
aliceExample
	| events |
	events _ self eventsFromString: self aliceExampleString.
	events do: [ :each | each pitchBy: 0.63489].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/28/1999 03:05'!
aliceExampleFemale
	| events |
	events _ self eventsFromString: self aliceExampleString.
	events do: [ :each | each pitchBy: 1.3].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/14/1999 01:09'!
aliceExampleMale
	| events |
	events _ self eventsFromString: self aliceExampleString.
	events do: [ :each | each pitchBy: 0.63489].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/29/1999 04:36'!
aliceShortExample
	| events |
	events _ self eventsFromString: self aliceShortExampleString.
	events do: [ :each | each pitchBy: 1.3].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 12/9/1999 02:09'!
aliceShortExampleMale
	| events |
	events _ self eventsFromString: self aliceShortExampleString.
	events do: [ :each | each pitchBy: 0.4].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 7/27/1999 01:47'!
mbrolaExample
	^ self eventsFromString:
'_ 50
E 40 0 102
m 50
b 50
r 30
@U 80 5 119 35 126 70 140
l 50
@ 50 50 173
w 100 75 133
V 30 85 114
z 60 75 101
d 60
@ 40
v 40 85 105
E 60 75 121
l 50 70 121
@ 60 60 150
p 50
d 70
b 70 0 90
AI 130 85 101
T 70
j 50 0 180
E 120 0 185 95 131
r 40
i 90 85 135
d 80 60 134
@ 50 0 119 50 114
t 70
w 70 10 117 65 127 85 115
A 180 0 102 55 91 95 85
_ 100
_ 100
I 80 18 111
t 80
s 50
@ 30
s 70
p 80
i 80 25 171 85 200
tS 110
s 70
I 30 35 112
n 40
T 80
@ 40 85 108
s 80
AI 130 80 115
z 70 90 125
r= 120 75 111
b 80
EI 80 95 133
z 70
d 50
A 40
n 30
D 60
@ 30 65 121
k 90
A 30
n 30 100 140
k 80
{ 70 5 170
t 70
@ 40 50 186
n 40 75 163
EI 100 90 173
S 130
@ 40
n 30 65 153
V 40
v 70 0 148
d 60
AI 130 5 112 80 109
f 110
@U 160 87 88
n 70
z 210 88 82
_ 80
_ 100
I 40 0 140
t 50
t 100
EI 60 50 221
k 70
s 80
@ 30 0 190
l 50
I 30 65 180
s 110
t 70
V 50 20 171
v 50
f 90
@U 140 25 157
n 30
i 60 66 160
m 50 60 130
z 80
@ 40 62 78
z 80
I 70 78 134
n 50 70 163
p 120
U 90 15 119 75 98
t 90
_ 140
_ 100
t 60 0 111
u 50 80 119
g 70
E 50 90 145
D 50
r= 50 40 139 90 163
w 90
I 30 15 114
D 50
_ 40
p 50
r 30 0 102
@ 30 65 110
s 120
A 50 90 148
d 80
I 50 50 178
k 80
I 50 66 167
n 60
f 50
r= 50 90 125
m 50
EI 140 95 96
S 140
@ 80 35 168
n 100 95 142
_ 190
{ 90 0 133
n 30
d 30
p 80
r 40
@ 40 0 97 65 103
d 70
j 20
u 60 65 150
s 90
I 30 50 210
z 50
s 140
p 70
i 130 0 138 95 98
tS 160
{ 70 0 127
t 50
D 50
@ 30 15 93
s 140
{ 30 35 127
m 50
p 70
l 50
I 30 35 184
N 70
f 70
r 60
i 90 30 125
k 40
w 30
@ 30 15 185
n 30
s 100
i 50 20 148 70 142
V 30 5 148
v 40
D 80
@ 40 25 106
d 80
AI 150 95 115
f 90
@U 130 95 114
n 70
d 80
EI 80 80 137
4 50
@ 30
b 100
EI 120 95 78
s 210
_ 80'! !

!PHOReader class methodsFor: 'examples' stamp: 'len 7/27/1999 01:48'!
pushExample
	^ self eventsFromString:
'_  60 0 137
p  50 100 137
u  110 90 137
S  100 10 121
D  90
@  70 57 114 100 102
s  70
t  50
A 100 57 121 64 121
r 40
t  50
b  110
V  140 21 117 57 100 92 100
_ 3
n  130 25 102 50 105
t  60
u  70 28 129 71 111
b  70
i  70 50 102
g  20 10 100 80 102
i  130 25 117 66 114
n  260 3 111 23 105 42 97 61 93 73 93
_  140 92 93 100 100'! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/16/1999 01:47'!
pushExampleFemale
	| events |
	events _ self pushExample.
	events do: [ :each | each pitchBy: 1.93489].
	^ events! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/26/1999 23:47'!
pushShortExample
	^ self eventsFromString:
'i  70 50 102
g  20 10 100 80 102
i  130 25 117 66 114
n  260 3 111 23 105 42 97 61 93 73 93'.
"
'_ 3
n  130 25 102 50 105
t  60
u  70 28 129 71 111
b  70
i  70 50 102
g  20 10 100 80 102
i  130 25 117 66 114
n  260 3 111 23 105 42 97 61 93 73 93
_  140 92 93 100 100'"! !

!PHOReader class methodsFor: 'examples' stamp: 'len 7/27/1999 01:48'!
xmasExample
	^ self eventsFromString:
'_  120 0 105
m  60 33 105
E  70 42 102
r  50 20 108
I  50 20 125 100 142
k  100
r  50 80 137
I  50 80 121
s  70
m  50
@  90 33 111 88 108
s  90
{  70
n  100 50 105 100 97
d  50 80 93
h  50 20 93
{  50 20 102 60 114
p  50
i  50 100 125
n  60 83 121
u 100
j  130 7 121 23 121 100 108
r=  250 41 102 83 97
_  210 95 86 100 100'
! !

!PHOReader class methodsFor: 'examples' stamp: 'len 9/29/1999 02:56'!
xmasKidExample
	| events |
	events _ self xmasExample.
	events do: [ :each | each pitchBy: 1.6].
	^ events! !


!PHOReader class methodsFor: 'examples-private' stamp: 'len 7/27/1999 01:45'!
aliceExampleString
	^ '_ 48 0 222
{ 80 40 222 90 235
l 72 44 250 66 250
I 80
s 88
w 40 80 235
@ 40 80 210
z 64 12 210 50 181 75 181
b 72 33 173 88 181
I 56 71 181
g 56 28 153 100 166
I 64 62 160
n 40 60 160
I 88 27 166
N 40 100 166
t 80 20 160
U 40 40 181 80 166
g 48 100 166
E 104 38 153 76 137
t 56
v 48
E 72 11 166 66 153
r 88 18 160 63 166
i 112.8 14 166 49 173 77 173
t 104.8 98 137
AI 178.4 21 148 70 148 92 137
r= 108 25 137 62 133 98 129
d 49
@ 59.2 6 137 73 133
v 46
s 89
I 40 40 173
t 40 40 166
I 64 62 166
N 72 22 160 77 153
b 40 40 142 100 153
AI 136 29 142
h 80 30 142 100 148
r= 64 62 153 100 153
s 125
I 64 37 210 100 190
s 72 11 190
t 72
r= 104 30 160 69 142
O 72
n 40 40 142
D 40 40 153
@ 40 40 160
b 40 40 153 100 160
{ 216 18 166 51 166 70 153 88 137 100 133
n 40
k 56
_ 144 94 129 100 100
_ 100 0 137
{ 40
n 88 9 137 36 137 72 142
t 40 40 137 60 133
@ 48 16 148 100 148
h 40 100 142
{ 48 83 148
v 48 16 142
I 56 14 148 85 137
N 56 71 137
n 64 37 137 100 142
O 96 41 137 83 153 100 153
T 64
I 56 42 166
N 64 12 160 75 153
t 80 30 142
@ 56 28 166 100 148
d 56 71 137
u 160 15 160 45 160 70 153 95 142
w 136 23 137 58 137 88 148
A 112 21 153 57 160 92 173
n 64 50 166 62 153
s 56
O 48 50 153
r 40 40 142 100 133
t 130
w 56 57 166
AI 168 9 173 33 166 57 153 80 148 100 148
s 88
_ 300
S 92
i 48 50 235
h 40 40 235 100 250
@ 40 60 210 100 210
d 40
p 40
i 150
k 56
d 56
I 72 44 222 100 210
n 40 80 210
t 56
@ 48 83 181
D 40 39 173 99 173
@ 39.2 100 160
b 104 30 153 100 153
U 112 35 153 85 153 100 153
k 72
h 56
r= 48 50 200
s 136 5 181
I 64 25 166 87 142
s 56 14 137
t 56
r= 56 28 173 100 173
w 56 71 166
@ 56 42 181
z 48 16 181 50 173
r 120 73 181
i 104 7 190 69 190 100 181
d 40
I 112 7 166 50 137 85 129
N 80 30 114 50 111
_ 56 83 111 100 100
_ 140 0 153
b 40 60 153 100 153
U 64 75 148
4 40 80 148 100 142
I 40
t 40 40 148
h 40 40 142
{ 48 33 137
d 136 5 133 11 129 88 148
n 48 66 148
@U 144 16 160 44 173 72 190 100 210
p 80
I 96 75 190 100 181
k 56
tS 96
r= 104 30 190 69 181
z 72 11 181 33 173
O 40 60 173
r 64 37 160 100 153
k 72 11 153
O 152 63 190 89 200
n 40 60 181
v 40 60 166
r= 40 60 160
s 104 15 142
EI 120 20 190 53 190 86 210
S 104 7 210
@ 56 28 181 100 160
n 64 62 153 100 137
z 40
I 56 14 148 85 133
n 40 100 133
I 96 41 129 83 125
t 80
_ 64 85 125 100 100
_ 140 0 222
{ 56 14 222 28 220 42 235 85 235 100 220
n 96 33 222 75 235 100 235
w 48 33 222 66 250
O 90
4 48 50 222 66 252
I 48 33 250
z 64 12 235 62 210
D 56
@ 56 71 190
j 104 23 166 61 166 100 181
u 112 35 190 42 210 71 210 100 210
s 150
@ 56 42 200
v 40 20 160 60 148
@ 64 25 153 87 142
b 136 23 133 47 137
U 96 8 160 50 173 91 181 100 181
k 56
_ 56
T 56
O 119.2 20 166 53 142 80 137
4 40 20 133
{ 136 5 137 35 129 64 125
l 48 16 125 100 133
I 80 50 153 100 181
s 120
_ 40
_ 40
w 88 9 166 54 142
I 40 40 142
D 40 40 137
aU 62.4 38 137 63 133
t 80
p 55
I 72 16 210 38 210
k 40
tS 75.2 99 173
r= 136 29 160 64 160 76 166
z 48
O 44 90 185
r 60 6 198 46 200 72 190
k 40
O 81.6 12 148 61 133
n 40 20 133
v 48 33 133
r= 56 28 181 100 181
s 120 6 173
EI 136 11 222 41 210 94 210
S 120
@ 128 12 190 43 148 75 137
n 64 12 129 50 125
_ 80 88 125 100 100'! !

!PHOReader class methodsFor: 'examples-private' stamp: 'len 7/27/1999 01:46'!
aliceShortExampleString
	^ '_ 48 0 222
{ 80 40 222 90 235
l 72 44 250 66 250
I 80
s 88
w 40 80 235
@ 40 80 210
z 64 12 210 50 181 75 181
b 72 33 173 88 181
I 56 71 181
g 56 28 153 100 166
I 64 62 160
n 40 60 160
I 88 27 166
N 40 100 166
t 80 20 160
U 40 40 181 80 166
g 48 100 166
E 104 38 153 76 137
t 56
v 48
E 72 11 166 66 153
r 88 18 160 63 166
i 112.8 14 166 49 173 77 173
t 104.8 98 137
AI 178.4 21 148 70 148 92 137
r= 108 25 137 62 133 98 129
d 49
@ 59.2 6 137 73 133
v 46
s 89
I 40 40 173
t 40 40 166
I 64 62 166
N 72 22 160 77 153
b 40 40 142 100 153
AI 136 29 142
h 80 30 142 100 148
@ 60 50 150
r= 64 62 153 100 153
s 125
I 64 37 210 100 190
s 72 11 190
t 72
r= 104 30 160 69 142
O 72
n 40 40 142
D 40 40 153
@ 40 40 160
b 40 40 153 100 160
{ 216 18 166 51 166 70 153 88 137 100 133
n 40
k 56
_ 144 94 129 100 100'! !
